// fragment shader for plugin textures

// uniforms - note they must be used if they are to be accessed in the calling program

uniform int   Clip1;
uniform vec3  MaterialC;              // Material second colour
uniform vec3  ScalingV;
uniform float SpecularPower;          // specularity  (range 10 - 80 )
uniform float SpecularContribution;   // mix of specular contribution - turns on/off spec 0.0/1.0
uniform float MixRatio;               // Reflection Fraction

uniform int   ShaderID;              
uniform float ShaderParameter;        // Shader Parameter

uniform vec3 AntiNormal1;        // Transform matrix for normal to undo camera
uniform vec3 AntiNormal2;        // Connection to a mat3 uniform  - does not work for some reason
uniform vec3 AntiNormal3;

uniform sampler3D NoiseMap;           // Map for 3D noise
uniform sampler2D RefMap;             // Reflection sampler
uniform sampler2D NormalMap;          // Map for bump mapping

const float DiffuseContribution=1.0;
const vec3  Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3  Yunitvec = vec3 (0.0, 1.0, 0.0);


varying vec3 tnorm;       // interpolated surface normal
varying vec4 PP;          // vertex global coord
varying vec3 texpos;      // shader coord relative to material shader axis

varying vec3 wvec;
varying vec3 uvec;
varying vec3 vvec;

void CalcLight(in vec3, in vec3, in vec3, inout vec4, inout vec4); // Light function (External)

#define ROOT32 0.8660254
#define ROOT3   1.732050

vec3 ppp[22];

void SetUpPPP(void){
 ppp[ 0] =vec3(0.0,0.25,0.5);                     
 ppp[ 1] =vec3(0.0,0.75,0.5);
 ppp[ 2] =vec3(ROOT32/2.0,0.0,0.5);
 ppp[ 3] =vec3(ROOT32/2.0,0.5,0.5);
 ppp[ 4] =vec3(ROOT32/2.0,1.0,0.5);
 ppp[ 5] =vec3(ROOT32,0.25,0.5);
 ppp[ 6] =vec3(ROOT32,0.75,0.5);
}

void main(void){
    SetUpPPP();
    if(Clip1 == 1){  // Use this as alternative to use the buid in clipping to make ATI/nVidia Compatible
      float d=dot(PP.xyz,gl_ClipPlane[0].xyz) + gl_ClipPlane[0].w;
      if( d < 0.0)discard;
    }
    // compute the fragment position in eye coordinates
    vec3 ecPosition = vec3(PP);
    // compute a unit vector in direction of viewing position
    vec3 viewVec    = normalize(-ecPosition);
    // must re-normalize interpolated vector to get Phong shading
    vec3 vnorm      = normalize(tnorm);
 
    // Prepare for reflection using pseudo environment map
    vec3  reflectDir = reflect(ecPosition,vnorm);
    mat3 AntiNormal; 
    AntiNormal[0]=AntiNormal1;  // Must do assignment in this way on ATI hardware
    AntiNormal[1]=AntiNormal2;
    AntiNormal[2]=AntiNormal3;
    reflectDir=AntiNormal*reflectDir;
    vec2 index;
    index.y = dot(normalize(reflectDir), Yunitvec);
    reflectDir.y = 0.0;
    index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;
    if (reflectDir.z >= 0.0)index = (index + 1.0) * 0.5;
    else {
        index.t = (index.t + 1.0) * 0.5;
        index.s = (-index.s) * 0.5 + 1.0;
     }
    vec3 envColor = vec3 (texture2D(RefMap, index)); 

    // make the bumps (1 bump centred inside each texture cube)
    int ii;
    //int id=0;
    vec3 p,pnn=gl_Color.rgb;

    p = texpos*ScalingV;
    p.x /= ROOT32;
    p.z = mod(3.0*p.z,2.0*3.14159);
    p.x += 0.11*sin(p.z);
    p.y += 0.15*cos(p.z);
    p.z=0.5;
    p.x = mod(p.x,ROOT32);
    p.y = mod(p.y,1.0);
    

    float dl,zb=2.0;
    for(ii=0; ii<7 ;ii++){
      if((dl=length(p-ppp[ii])) < 0.25){ 
        if(dl < zb){
          //id=1;
          zb=dl;
          pnn=MaterialC;
          if(dl > 0.2){
            pnn=mix(pnn,gl_Color.rgb,(dl-0.15)/0.1);
          }
        }
      }
    }
    vec4 surface_colour=vec4(pnn,1.0);
    //if(id == 0)discard;
    // Calculate the Phong lighting
    vec4 diffuse=vec4(0.0,0.0,0.0,1.0);
    vec4 spec=vec4(0.0,0.0,0.0,0.0);
    CalcLight(ecPosition,viewVec,vnorm,diffuse,spec);

    // now make the fragment
    gl_FragColor = mix(surface_colour * DiffuseContribution * diffuse, vec4(envColor,1.0),MixRatio)
                 + spec * SpecularContribution;
}
