// CRSPOTS fragment shader for plugin textures

// uniforms - note they must be used if they are to be accessed in the calling program

uniform int   Clip1;
uniform vec3  MaterialC;              // Material second colour
uniform vec3  ScalingV;
uniform float SpecularPower;          // specularity  (range 10 - 80 )
uniform float SpecularContribution;   // mix of specular contribution - turns on/off spec 0.0/1.0
uniform float MixRatio;               // Reflection Fraction

uniform int   ShaderID;               
const float BumpSlope=1.0;
const float RidgeThickness=0.1;

uniform vec3 AntiNormal1;        // Transform matrix for normal to undo camera
uniform vec3 AntiNormal2;        // Connection to a mat3 uniform  - does not work for some reason
uniform vec3 AntiNormal3;

uniform sampler2D NoiseData;          // RandomNumbers
uniform sampler3D NoiseMap;           // Map for 3D noise
uniform sampler2D RefMap;             // Reflection sampler
uniform sampler2D NormalMap;          // Map for bump mapping

const float DiffuseContribution=1.0;
const vec3  Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3  Yunitvec = vec3 (0.0, 1.0, 0.0);


varying vec3 tnorm;       // interpolated surface normal
varying vec4 PP;          // vertex global coord
varying vec3 texpos;      // shader coord relative to material shader axis

varying vec3 wvec;
varying vec3 uvec;
varying vec3 vvec;

void CalcLight(in vec3, in vec3, in vec3, inout vec4, inout vec4); // Light function (External)

#define PATTERN(x,y,z)   ( ((z)+127.0)*511.0 +  \
                           ((x)+127.0)*255.0 +  \
                           ((y)+127.0))

//#define ScaleD 1.0;
#define ScaleD 2.0;

void main(void){
    vec3 ppp[27];
    float bradius,lradius;
    vec3 vclr[27];
    ppp[ 0]=vec3( 0.0,0.0,0.0);
    ppp[ 1]=vec3( 1.0,0.0,0.0);
    ppp[ 2]=vec3( 1.0,1.0,0.0);
    ppp[ 3]=vec3( 0.0,1.0,0.0);
    ppp[ 4]=vec3(-1.0,1.0,0.0);
    ppp[ 5]=vec3(-1.0,0.0,0.0);
    ppp[ 6]=vec3(-1.0,-1.0,0.0);
    ppp[ 7]=vec3( 0.0,-1.0,0.0);
    ppp[ 8]=vec3( 1.0,-1.0,0.0);
    ppp[ 9]=vec3( 0.0,0.0,-1.0);
    ppp[10]=vec3( 1.0,0.0,-1.0);
    ppp[11]=vec3( 1.0,1.0,-1.0);
    ppp[12]=vec3( 0.0,1.0,-1.0);
    ppp[13]=vec3(-1.0,1.0,-1.0);
    ppp[14]=vec3(-1.0,0.0,-1.0);
    ppp[15]=vec3(-1.0,-1.0,-1.0);
    ppp[16]=vec3( 0.0,-1.0,-1.0);
    ppp[17]=vec3( 1.0,-1.0,-1.0);
    ppp[18]=vec3( 0.0,0.0,1.0);
    ppp[19]=vec3( 1.0,0.0,1.0);
    ppp[20]=vec3( 1.0,1.0,1.0);
    ppp[21]=vec3( 0.0,1.0,1.0);
    ppp[22]=vec3(-1.0,1.0,1.0);
    ppp[23]=vec3(-1.0,0.0,1.0);
    ppp[24]=vec3(-1.0,-1.0,1.0);
    ppp[25]=vec3( 0.0,-1.0,1.0);
    ppp[26]=vec3( 1.0,-1.0,1.0);
    if(Clip1 == 1){  // Use this as alternative to use the buid in clipping to make ATI/nVidia Compatible
      float d=dot(PP.xyz,gl_ClipPlane[0].xyz) + gl_ClipPlane[0].w;
      if( d < 0.0)discard;
    }
    // compute the fragment position in eye coordinates
    vec3 ecPosition = vec3(PP);
    // compute a unit vector in direction of viewing position
    vec3 viewVec    = normalize(-ecPosition);
    // must re-normalize interpolated vector to get Phong shading
    vec3 vnorm      = normalize(tnorm);
 
    // Prepare for reflection using pseudo environment map
    vec3  reflectDir = reflect(ecPosition,vnorm);
    mat3 AntiNormal; 
    AntiNormal[0]=AntiNormal1;  // Must do assignment in this way on ATI hardware
    AntiNormal[1]=AntiNormal2;
    AntiNormal[2]=AntiNormal3;
    reflectDir=AntiNormal*reflectDir;
    vec2 index;
    index.y = dot(normalize(reflectDir), Yunitvec);
    reflectDir.y = 0.0;
    index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;
    if (reflectDir.z >= 0.0)index = (index + 1.0) * 0.5;
    else {
        index.t = (index.t + 1.0) * 0.5;
        index.s = (-index.s) * 0.5 + 1.0;
     }
    vec3 envColor = vec3 (texture2D(RefMap, index)); 

    // make the bumps (1 bump centred inside each texture cube)
    float pat,id= -1.0;
    vec3 vclri,clr;
    vec3 p,pf,pl,nDelta,pnn=vec3(0.0);
    vec2 c;
    pf = texpos*ScalingV;                    
//pf *= 4.0;
    p=fract(pf);
    float dl,sfactor,nw,nu,nv,zb=4.0,zl=4.0,radius;
    pf=floor(pf);
    bradius=0.5; lradius=0.02;
//    bradius=1.0; lradius=0.07;  // overlapping
//bradius=0.25;
//bradius=1.0;
    int ii; for(ii=0;ii<27;ii++){
      pat=PATTERN(ppp[ii].x+pf.x,ppp[ii].y+pf.y,ppp[ii].z+pf.z);
      pat=mod(pat,262144.0);
      c.y=floor(pat/512.0)/512.0;
      c.x=(pat-(floor(pat/512.0)*512.0))/512.0;
      vclri=texture2D(NoiseData,c).rgb;
      radius=bradius+lradius*(vclri[1]-0.5);   
      pl=ppp[ii]+vclri*ScaleD;
      pl=p-pl;
      if((dl=length(pl)) < radius){   // spot radius  need to randomize  (0.3 min 0.5 (mid) 0.7 overlaps)
        if(dl < zb){
          zl=zb;
          zb=dl;
          id=float(ii);
          clr=vclri;
          pnn=ppp[ii];
        }
      }
    }
    vec4 surface_colour=gl_Color;
    if(id >= 0.0){
      surface_colour=vec4(clr,1.0);
//    p=p-pnn;
//    sfactor=0.5*BumpSlope;   // bump slope;
//    p=normalize(p);  // to ensure planar edge

//    dl=abs(zl-zb);
//    if(dl < 0.05){ // edge
//    if(zb > 0.45){ // edge
//      p=vec3(0.0);
//      surface_colour=vec4(MaterialC,1.0);
//    }
//    else 
//      surface_colour=vec4(clr,1.0); 
//    nw=abs(dot(vnorm,wvec));
//    nu=abs(dot(vnorm,uvec));
//    nv=abs(dot(vnorm,vvec));
//    c=p.xy;
//    nDelta = nw*(c.x*uvec + c.y*vvec);
//    c=p.yz; c.y *= -1.0;    // note inversion 
//    nDelta = nDelta + nu*(c.x*vvec + c.y*wvec);
//    c=p.zx; c.x *= -1.0;   // note swizzle z/x and inversion
//    nDelta = nDelta + nv*(c.x*wvec + c.y*uvec);
 //   if(Type == 0)vnorm -= nDelta*sfactor;  
 //   else         vnorm += nDelta*sfactor;  
 //   vnorm=normalize(vnorm);
    }

    // Calculate the Phong lighting
    vec4 diffuse=vec4(0.0,0.0,0.0,1.0);
    vec4 spec=vec4(0.0,0.0,0.0,0.0);
    CalcLight(ecPosition,viewVec,vnorm,diffuse,spec);  //diffuse=1.0; spec=0.0;

    // now make the fragment
    gl_FragColor = mix(surface_colour * DiffuseContribution * diffuse, vec4(envColor,1.0),MixRatio)
                 + spec * SpecularContribution;
}
