// plates - fragment shader for materials with 3d noise

// uniforms - note they must be used if they are to be accessed in the calling program

uniform int   Clip1;
uniform vec3  MaterialC;              // Material second colour
uniform float SpecularPower;          // specularity  (range 10 - 80 )
uniform float SpecularContribution;   // mix of specular contribution - turns on/off spec 0.0/1.0
uniform int   ShaderID;               // shader ID
uniform int   ShaderInternal;         // TRUE (1) if Internal Shader
uniform float ShaderParameter;        // Shader Parameter
uniform float MixRatio;               // Reflection Fraction

uniform vec3 ScalingV;

uniform vec3 AntiNormal1;        // Transform matrix for normal to undo camera
uniform vec3 AntiNormal2;        // Connection to a mat3 uniform  - does not work for some reason
uniform vec3 AntiNormal3;

uniform sampler2D NoiseData;          // RandomNumbers
uniform sampler3D NoiseMap;           // Map for 3D noise
uniform sampler2D RefMap;             // Reflection sampler
uniform sampler2D NormalMap;          // Map for bump mapping

const float dhue = 0.05;
const float dval = 0.3;
const int   bDirty=1;

const float DiffuseContribution=1.0;
const vec3  Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3  Yunitvec = vec3 (0.0, 1.0, 0.0);

const float t=0.4,dt=0.18;

varying vec3 tnorm;       // interpolated surface normal
varying vec4 PP;          // vertex global coord
varying vec3 texpos;      // shader coord relative to material shader axis

varying vec3 wvec;
varying vec3 uvec;
varying vec3 vvec;

#define PATTERN(x,y,z)   ( ((z)+127.0)*511.0 +  \
                           ((x)+127.0)*255.0 +  \
                           ((y)+127.0))


void CalcLight(in vec3, in vec3, in vec3, inout vec4, inout vec4); // Light function (External)

void rgb2hsv(in float r, in float g, in float b, out float h, out float s, out float v){
 float maxcol,mincol;
 maxcol=max(r,max(g,b));
 mincol=min(r,min(g,b));
 v=maxcol;
 if(maxcol == mincol){s=0.0; h=0.0;}
 else{
  s=(maxcol-mincol)/maxcol;
  if(mincol == b)
    h = 120.0*(g-mincol)/(r+g-2.0*mincol);
  else if(mincol == r)
    h = 120.0*(1.0+(b-mincol)/(b+g-2.0*mincol));
  else
    h = 120.0*(2.0+(r-mincol)/(r+b-2.0*mincol));
 }
}

void hsv2rgb(in float hh, in float ss, in float vv,
             out float rr, out float gg, out float bb){
 float maxcol,mincol;
 mincol=vv*(1.0-ss);
 if(hh <= 120.0){bb=mincol;
   if(hh <=  60){rr=vv; gg=mincol+hh*(vv-mincol)/(120.0-hh);}
   else         {gg=vv; rr=mincol+(120.0-hh)*(vv-mincol)/hh;}
 }
 else if( hh <= 240){rr=mincol;
   if(hh <= 180){gg=vv; bb=mincol+(hh-120.0)*(vv-mincol)/(240.0-hh);}
   else         {bb=vv; gg=mincol+(240.0-hh)*(vv-mincol)/(hh-120.0);}
 }
 else {gg=mincol;
   if(hh <= 300){bb=vv; rr=mincol+(hh-240.0)*(vv-mincol)/(360.0-hh);}
   else         {rr=vv; bb=mincol+(360.0-hh)*(vv-mincol)/(hh-240.0);}
 }
}

void main(void){
    if(Clip1 == 1){  // Use this as alternative to use the buid in clipping to make ATI/nVidia Compatible
      float d=dot(PP.xyz,gl_ClipPlane[0].xyz) + gl_ClipPlane[0].w;
      if( d < 0.0)discard;
    }
    // compute the fragment position in eye coordinates
    vec3 ecPosition = vec3(PP);
    // compute a unit vector in direction of viewing position
    vec3 viewVec    = normalize(-ecPosition);
    // must re-normalize interpolated vector to get Phong shading
    vec3 vnorm      = normalize(tnorm);
 
    // Prepare for reflection using pseudo environment map
    vec3  reflectDir = reflect(ecPosition,vnorm);
    mat3 AntiNormal; 
    AntiNormal[0]=AntiNormal1;  // Must do assignment in this way on ATI hardware
    AntiNormal[1]=AntiNormal2;
    AntiNormal[2]=AntiNormal3;
    reflectDir=AntiNormal*reflectDir;
    vec2 index;
    index.y = dot(normalize(reflectDir), Yunitvec);
    reflectDir.y = 0.0;
    index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;
    if (reflectDir.z >= 0.0)index = (index + 1.0) * 0.5;
    else {
        index.t = (index.t + 1.0) * 0.5;
        index.s = (-index.s) * 0.5 + 1.0;
     }
    vec3 envColor = vec3 (texture2D(RefMap, index)); 
//    vec3 envColor = vec3 (texture3D(RefMap3D, vec3(index,0.5))); 

    vec4 surface_colour=gl_Color;
    
    vec3 pp=texpos*ScalingV*1.0;
    pp.z *= 2.0;
    
    vec3 vclri;
    vec2 c;
    float pat;
    pat=PATTERN(floor(pp.x),floor(pp.y),floor(pp.z));
    pat=mod(pat,262144.0);
    c.y=floor(pat/512.0)/512.0;
    c.x=(pat-(floor(pat/512.0)*512.0))/512.0;
    vclri=texture2D(NoiseData,c).rgb;
    if(ShaderID == 1){ // random colour 
      surface_colour = vec4(vclri.r,vclri.g,vclri.b,gl_Color.a);
    }
    else{ // random alteration to hue
      float hue,sat,val;
      rgb2hsv(surface_colour.r,surface_colour.g,surface_colour.b,hue,sat,val);
      hue += (vclri[0]-0.5)*360.0*dhue;
      if(hue < 0.0)   hue += 360.0;
      if(hue > 360.0) hue -= 360.0;
      val += dval*(vclri[1]-0.5); val=clamp(val,0.0,1.0);  
      hsv2rgb(hue,sat,val,surface_colour.r,surface_colour.g,surface_colour.b);
    }
    
    if(bDirty == 1){ // dirty
      vec3 noisef;
      noisef=texture3D(NoiseMap,pp*(0.8)).rgb;
      surface_colour -= vec4(noisef,0.0);
    }
    
    pp=fract(pp);  // put in the grooves between the plates
    float dd = 0.05;
    vec3 vt=vec3(0.0);
    if     (pp.x < dd    )vt =  uvec;
    else if(pp.x > 1.0-dd)vt = -uvec;
    else if(pp.y < dd    )vt =  vvec;
    else if(pp.y > 1.0-dd)vt = -vvec;
    else if(pp.z < dd    )vt =  wvec;
    else if(pp.z > 1.0-dd)vt = -wvec;
    vnorm += 0.45*vt;
    normalize(vnorm);

    // Calculate the Phong lighting
    vec4 diffuse=vec4(0.0,0.0,0.0,1.0);
    vec4 spec=vec4(0.0,0.0,0.0,0.0);
    CalcLight(ecPosition,viewVec,vnorm,diffuse,spec);
 
    // now make the fragment
    gl_FragColor = mix(surface_colour * DiffuseContribution * diffuse, vec4(envColor,1.0),MixRatio)
                 + spec * SpecularContribution;
}
