// HONEY fragment shader for plugin textures

// uniforms - note they must be used if they are to be accessed in the calling program

uniform int   Clip1;
uniform vec3  MaterialC;              // Material second colour
uniform vec3  ScalingV;
uniform float SpecularPower;          // specularity  (range 10 - 80 )
uniform float SpecularContribution;   // mix of specular contribution - turns on/off spec 0.0/1.0
uniform float MixRatio;               // Reflection Fraction

uniform int   ShaderID;               // Type of bump
uniform int   Type;                   // raised/sunken
uniform int   ShaderInternal;         // TRUE (1) if Internal Shader
uniform float ShaderParameter;        // Shader Parameter
uniform float BumpSlope;
uniform float RidgeThickness;
uniform float EdgeThickness;

uniform vec3 AntiNormal1;        // Transform matrix for normal to undo camera
uniform vec3 AntiNormal2;        // Connection to a mat3 uniform  - does not work for some reason
uniform vec3 AntiNormal3;

uniform sampler3D NoiseMap;           // Map for 3D noise
uniform sampler2D RefMap;             // Reflection sampler
uniform sampler2D NormalMap;          // Map for bump mapping

const float DiffuseContribution=1.0;
const vec3  Xunitvec = vec3 (1.0, 0.0, 0.0);
const vec3  Yunitvec = vec3 (0.0, 1.0, 0.0);


varying vec3 tnorm;       // interpolated surface normal
varying vec4 PP;          // vertex global coord
varying vec3 texpos;      // shader coord relative to material shader axis

varying vec3 wvec;
varying vec3 uvec;
varying vec3 vvec;

void CalcLight(in vec3, in vec3, in vec3, inout vec4, inout vec4); // Light function (External)

#define ROOT32 0.8660254
#define ROOT3   1.732050

vec3 ppp[35];

void SetUpPPP(void){
 ppp[ 0] =vec3(0.25,0.0,      0.0);                     
 ppp[ 1] =vec3(0.75,0.0,      0.0);
 ppp[ 2] =vec3(0.0, ROOT32/2.0, 0.0);
 ppp[ 3] =vec3(0.5, ROOT32/2.0, 0.0);
 ppp[ 4] =vec3(1.0, ROOT32/2.0, 0.0);
 ppp[ 5] =vec3(0.25,ROOT32,   0.0);
 ppp[ 6] =vec3(0.75,ROOT32,   0.0);
 ppp[ 7] =vec3(-0.25,0.0,     0.0);
 ppp[ 8] =vec3( 1.25,0.0,     0.0);
 ppp[ 9] =vec3(-0.25,ROOT32,  0.0);
 ppp[10] =vec3( 1.25,ROOT32,  0.0);
 ppp[11] =vec3(-0.25,0.0,    ROOT32);
 ppp[12] =vec3( 1.25,0.0,    ROOT32);
 ppp[13] =vec3(-0.25,ROOT32, ROOT32);
 ppp[14] =vec3( 1.25,ROOT32, ROOT32);
 ppp[15] =vec3(-0.25,1.0/2.0/ROOT3,ROOT32/2.0);
 ppp[16] =vec3( 1.75,1.0/2.0/ROOT3,ROOT32/2.0);
 ppp[17] =vec3(0.0,1.0/2.0/ROOT3+ROOT32,ROOT32/2.0);
 ppp[18] =vec3(0.5,1.0/2.0/ROOT3+ROOT32,ROOT32/2.0);
 ppp[19] =vec3(1.0,1.0/2.0/ROOT3+ROOT32,ROOT32/2.0);
 ppp[20] =vec3(0.0, 1.0/2.0/ROOT3-ROOT32/2.0,ROOT32/2.0);
 ppp[21] =vec3(0.5, 1.0/2.0/ROOT3-ROOT32/2.0,ROOT32/2.0);
 ppp[22] =vec3(1.0, 1.0/2.0/ROOT3-ROOT32/2.0,ROOT32/2.0);
 ppp[23] =vec3(0.25,1.0/2.0/ROOT3,         ROOT32/2.0);
 ppp[24] =vec3(0.75,1.0/2.0/ROOT3,         ROOT32/2.0);
 ppp[25] =vec3(0.0, 1.0/2.0/ROOT3+ROOT32/2.0,ROOT32/2.0);
 ppp[26] =vec3(0.5, 1.0/2.0/ROOT3+ROOT32/2.0,ROOT32/2.0);
 ppp[27] =vec3(1.0, 1.0/2.0/ROOT3+ROOT32/2.0,ROOT32/2.0);
 ppp[28] =vec3(0.25,0.0,      ROOT32);
 ppp[29] =vec3(0.75,0.0,      ROOT32);
 ppp[30] =vec3(0.0, ROOT32/2.0, ROOT32);
 ppp[31] =vec3(0.5, ROOT32/2.0, ROOT32);
 ppp[32] =vec3(1.0, ROOT32/2.0, ROOT32);
 ppp[33] =vec3(0.25,ROOT32,   ROOT32);
 ppp[34] =vec3(0.75,ROOT32,   ROOT32);
}

void main(void){
    SetUpPPP();
    if(Clip1 == 1){  // Use this as alternative to use the buid in clipping to make ATI/nVidia Compatible
      float d=dot(PP.xyz,gl_ClipPlane[0].xyz) + gl_ClipPlane[0].w;
      if( d < 0.0)discard;
    }
    // compute the fragment position in eye coordinates
    vec3 ecPosition = vec3(PP);
    // compute a unit vector in direction of viewing position
    vec3 viewVec    = normalize(-ecPosition);
    // must re-normalize interpolated vector to get Phong shading
    vec3 vnorm      = normalize(tnorm);
 
    // Prepare for reflection using pseudo environment map
    vec3  reflectDir = reflect(ecPosition,vnorm);
    mat3 AntiNormal; 
    AntiNormal[0]=AntiNormal1;  // Must do assignment in this way on ATI hardware
    AntiNormal[1]=AntiNormal2;
    AntiNormal[2]=AntiNormal3;
    reflectDir=AntiNormal*reflectDir;
    vec2 index;
    index.y = dot(normalize(reflectDir), Yunitvec);
    reflectDir.y = 0.0;
    index.x = dot(normalize(reflectDir), Xunitvec) * 0.5;
    if (reflectDir.z >= 0.0)index = (index + 1.0) * 0.5;
    else {
        index.t = (index.t + 1.0) * 0.5;
        index.s = (-index.s) * 0.5 + 1.0;
     }
    vec3 envColor = vec3 (texture2D(RefMap, index)); 

    // make the bumps (1 bump centred inside each texture cube)
    int ii;
    vec3 p,nDelta,pnn=vec3(0.0),pll=vec3(0.0);
    vec2 c;
    p = texpos*ScalingV;
    p.y /= ROOT32;
    p.z /= ROOT32;
    p.x=mod(p.x,1.0);
    p.y=mod(p.y,ROOT32);
    p.z=mod(p.z,ROOT32);
    float dl,sfactor,nw,nu,nv,zb=2.0,zl=2.0;
    for(ii=0; ii<35 ;ii++){
      if((dl=length(p-ppp[ii])) < 0.6){ // 0.6  fairly arbitrary
        if(dl < zb){
          zl=zb; // second closest
          pll=pnn; 
          zb=dl;
          pnn=ppp[ii];
        }
      }
    }
    vec4 surface_colour=gl_Color;

    p=p-pnn;
    sfactor=0.5*BumpSlope;   // bump slope
    p=normalize(p);  // to ensure planar edge
    dl=abs(zl-zb);
    // if(dl < 0.05){  // RidgeThickness
    if(dl < RidgeThickness){ 
      p=vec3(0.0);
      surface_colour=vec4(MaterialC,1.0);
    }
    //else if(dl > 0.1)p=vec3(0.0); // EdgeThickness
    else if(dl > EdgeThickness)p=vec3(0.0);  
    nw=abs(dot(vnorm,wvec));
    nu=abs(dot(vnorm,uvec));
    nv=abs(dot(vnorm,vvec));
    c=p.xy;
    nDelta = nw*(c.x*uvec + c.y*vvec);
    c=p.yz; c.y *= -1.0;    // note inversion 
    nDelta = nDelta + nu*(c.x*vvec + c.y*wvec);
    c=p.zx; c.x *= -1.0;   // note swizzle z/x and inversion
    nDelta = nDelta + nv*(c.x*wvec + c.y*uvec);
    if(Type == 0)vnorm -= nDelta*sfactor;  
    else         vnorm += nDelta*sfactor;  
    vnorm=normalize(vnorm);

    // Calculate the Phong lighting
    vec4 diffuse=vec4(0.0,0.0,0.0,1.0);
    vec4 spec=vec4(0.0,0.0,0.0,0.0);
    CalcLight(ecPosition,viewVec,vnorm,diffuse,spec);

    // now make the fragment
    gl_FragColor = mix(surface_colour * DiffuseContribution * diffuse, vec4(envColor,1.0),MixRatio)
                 + spec * SpecularContribution;
}
